﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using System.Collections;
using System.Globalization;
using System.Data.Common;

namespace AavsoReportGenerator
{
    public partial class AavsoReportGenerator : Form
    {
        public AavsoReportGenerator()
        {
            InitializeComponent();
        }

        private void btnSelect_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog FolderBrowserDialog1 = new FolderBrowserDialog()
            {
                SelectedPath = this.txtDirectory.Text,
                ShowNewFolderButton = true,
                Description="Select folder with AAVSO report files"
            };

            if (FolderBrowserDialog1.ShowDialog() == DialogResult.OK)
            {
                this.txtDirectory.Text = FolderBrowserDialog1.SelectedPath;
            }
        }

        private void btnGenerate_Click(object sender, EventArgs e)
        {
            try
            {
                DateTime starttime = DateTime.Now;

                this.label_Status.Text = "Processing...";
                this.label_Status.ForeColor = Color.Orange;
                this.label_Status.Refresh();

                this.txtOutput.AppendText("Generating mass AAVSO report from multiple AAVSO reports.");
                this.txtOutput.AppendText(Environment.NewLine);
                if (Directory.Exists(this.txtDirectory.Text))
                {
                    this.txtOutput.AppendText("Directory Exists\n");

                    string[] files = Directory.GetFiles(this.txtDirectory.Text, this.txtWildcard.Text, SearchOption.TopDirectoryOnly);
                    string[] files_csv = Directory.GetFiles(this.txtDirectory.Text, "CsvReport*.csv", SearchOption.TopDirectoryOnly);
                    this.txtOutput.AppendText("AAVSO Report files found: " + files.Count() + "\n");
                    this.txtOutput.AppendText(Environment.NewLine);

                    if (files.Count() != 0)
                    {
                        //string outputFile = this.txtDirectory.Text + @"\" + this.txtFilename.Text;
                        string outputFile = Path.Combine(this.txtDirectory.Text, this.txtFilename.Text);
                        if (File.Exists(outputFile))
                        {
                            outputFile = Path.Combine(this.txtDirectory.Text,Path.GetFileNameWithoutExtension(outputFile)+DateTime.Now.Year + DateTime.Now.Month + DateTime.Now.Day + DateTime.Now.Hour + DateTime.Now.Minute + DateTime.Now.Second + DateTime.Now.Millisecond+@".txt");
                            this.txtOutput.AppendText("Supplied AAVSO report filename already exists.  Renaming to: " + outputFile);
                            this.txtOutput.AppendText(Environment.NewLine);
                        }
                        this.txtOutput.AppendText("Generating output file: " + outputFile);
                        this.txtOutput.AppendText(Environment.NewLine);
                        StreamWriter fileWriter = new StreamWriter(outputFile, true);

                        string outputFile_err = Path.Combine(this.txtDirectory.Text, Path.GetFileNameWithoutExtension(this.txtFilename.Text)+"error.txt");
                        if (File.Exists(outputFile_err))
                        {
                            //if (this.checkBox_CSVerrors.Checked || this.checkBox_EmptyAAVSO.Checked)
                            //{
                            string outputfile_err = Path.Combine(this.txtDirectory.Text, Path.GetFileNameWithoutExtension(outputFile_err) + DateTime.Now.Year + DateTime.Now.Month + DateTime.Now.Day + DateTime.Now.Hour + DateTime.Now.Minute + DateTime.Now.Second + DateTime.Now.Millisecond + @"_errors.txt");

                            //}
                        }
                        StreamWriter fileWriter_err = new StreamWriter(outputFile_err, true);
                        ArrayList fulloutputfile = new ArrayList();

                        this.txtOutput.AppendText("Checking Empty AAVSO files:");
                        this.txtOutput.AppendText(Environment.NewLine);
                        fileWriter_err.WriteLine("===========================");
                        fileWriter_err.WriteLine("Checking Empty AAVSO files:");
                        fileWriter_err.WriteLine("===========================");

                        for (int i = 0; i < files.Count(); i++)
                        {
                            this.txtOutput.AppendText("Processing file: "+files[i].ToString());
                            this.txtOutput.AppendText(Environment.NewLine);
                            //StreamReader file = new StreamReader(this.txtDirectory.Text + @"\" + files[i].ToString());
                            StreamReader file = new StreamReader(files[i].ToString());

                            string line;
                            string lastline="abc";
                            while ((line = file.ReadLine()) != null)
                            {
                                if (i == 0)
                                {
                                    //Eerste file, kopieer alle lijnen
                                    //fileWriter.WriteLine(line);
                                    fulloutputfile.Add(line);
                                    this.txtOutput.AppendText(line);
                                    this.txtOutput.AppendText(Environment.NewLine);
                                }
                                else
                                {
                                    //Alle andere files, kopieer enkel de datalijnen
                                    if (line.Substring(0, 1) != "#")
                                    {
                                        //fileWriter.WriteLine(line);
                                        fulloutputfile.Add(line);
                                        this.txtOutput.AppendText(line);
                                        this.txtOutput.AppendText(Environment.NewLine);
                                    }
                                }
                                lastline = line;
                            }

                            if (this.checkBox_EmptyAAVSO.Checked && (lastline.Substring(0, 1) == "#"))
                                fileWriter_err.WriteLine("AAVSO EMPTY: " + Path.GetFileName(files[i].ToString()));

                            file.Close();
                            file.Dispose();
                            file = null;


                        } //end for

                        if (this.checkBox_CSVerrors.Checked)
                        {
                            this.txtOutput.AppendText("Checking CSV files:");
                            this.txtOutput.AppendText(Environment.NewLine);
                            fileWriter_err.WriteLine("===================");
                            fileWriter_err.WriteLine("Checking CSV files:");
                            fileWriter_err.WriteLine("===================");
                            for (int i = 0; i < files_csv.Count(); i++)
                            {
                                StreamReader file = new StreamReader(files_csv[i].ToString());

                                string line;
                                while ((line = file.ReadLine()) != null)
                                {
                                    string firstword = line.Split(' ')[0];
                                    if (firstword != "LesvePhotometry")
                                    {
                                        string[] split = line.Split(',');
                                        if (split[0] != "FileName")
                                        {
                                            if (split[20] == "NOK")
                                            {
                                                string errormessage = split[0] + ":" + split[1] + ":" + split[32];
                                                this.txtOutput.AppendText(errormessage);
                                                this.txtOutput.AppendText(Environment.NewLine);
                                                fileWriter_err.WriteLine(errormessage);
                                            }
                                        }
                                    }
                                }

                                file.Close();
                                file.Dispose();
                                file = null;
                            }
                        }

                        if (this.checkBox_doubleoptimization.Checked)
                        {
                            this.txtOutput.AppendText("Perform Double Optimization:");
                            this.txtOutput.AppendText(Environment.NewLine);
                            fileWriter_err.WriteLine("============================");
                            fileWriter_err.WriteLine("Perform Double Optimization:");
                            fileWriter_err.WriteLine("============================");
                            //check als er een lijn bestaat met bestaande object/filter combinatie met betere error
                            ArrayList detectedobjs = new ArrayList();
                            for (int i=0; i<fulloutputfile.Count;i++)
                            {
                                this.txtOutput.AppendText("Loop: "+i);
                                this.txtOutput.AppendText(Environment.NewLine);
                                if (fulloutputfile[i].ToString().Substring(0, 1) != "#")
                                {
                                    string[] sfirst = fulloutputfile[i].ToString().Split(',');
                                    int nrofdoubles = 0;
                                    //eerste iteratie om te hoeveel dubbeleobjecten er zijn
                                    for (int j = 0; j < fulloutputfile.Count; j++)
                                    {
                                        string[] strynext = fulloutputfile[j].ToString().Split(',');
                                        if ((strynext[0].Equals(sfirst[0])) && (strynext[4].Equals(sfirst[4])))
                                            nrofdoubles++;
                                    }
                                    if ((nrofdoubles > 1) && (nrofdoubles < 7))     //als geen 1 object en als geen tijdreeks (tijdreeks nemen we gewoon 7 datapunten minimum)
                                    {
                                        detectedobjs.Add(fulloutputfile[i].ToString().Split(',')[0]);
                                        do
                                        {
                                            this.txtOutput.AppendText("do: " + nrofdoubles);
                                            this.txtOutput.AppendText(Environment.NewLine);
                                            //fileWriter_err.WriteLine(fulloutputfile[i]);
                                            for (int j = 0; j < fulloutputfile.Count; j++)
                                            {
                                                //itereer om te zien of er dubbles zijn
                                                string[] strynext = fulloutputfile[j].ToString().Split(',');
                                                if ((strynext[0].Equals(sfirst[0])) && (strynext[4].Equals(sfirst[4])) && !sfirst[0].Substring(0, 1).Equals("!"))     //object en filter dezelfde en nog niet gevlagd
                                                {
                                                    //fileWriter_err.WriteLine(fulloutputfile[j]);
                                                    //check om te zien of het niét dezelfde lijn is, maw als de error én tijd niet hetzelfde is
                                                    double errorfirst = Double.Parse(sfirst[3]);
                                                    double errortry = Double.Parse(strynext[3]);
                                                    this.txtOutput.AppendText("do obj: " + strynext[0]);
                                                    this.txtOutput.AppendText(Environment.NewLine);
                                                    //nrofdoubles--;
                                                    if (!(strynext[3].Equals(sfirst[3])) || !strynext[1].Equals(sfirst[1]))
                                                    {
                                                        if (errorfirst > errortry)
                                                        {
                                                            //oorspronkelijke lijn heeft grotere error 
                                                            fulloutputfile[i] = "!" + fulloutputfile[i];      //vlag met een "!", dus niét schrijven
                                                        }
                                                        nrofdoubles--;
                                                    }
                                                }
                                            }
                                        } while (nrofdoubles > 1);
                                    }
                                } 
                            }
                            //Toon de gedetecteerde objecten in de errorfile
                            for (int i = 0; i < fulloutputfile.Count; i++)
                            {
                                string line = fulloutputfile[i].ToString();
                                bool todelete = false;
                                while (line.Substring(0,1).Equals("!"))
                                {
                                    line = line.Remove(0, 1);
                                    todelete = true;
                                }
                                string obj = line.Split(',')[0];
                                bool hit = false;
                                for (int j=0; j<detectedobjs.Count; j++)
                                {
                                    //fileWriter_err.WriteLine(detectedobjs[j]);
                                    if (obj.Equals(detectedobjs[j]))
                                        hit = true;
                                }
                                if (hit == true)
                                {
                                    if (todelete == true)
                                    {
                                        fileWriter_err.WriteLine("DEL: "+line);
                                    }
                                    else
                                    {
                                        fileWriter_err.WriteLine("     " + line);
                                    }

                                }

                            }

                        }

                        if (this.checkBox_useExclusionList.Checked)
                        {
                            this.txtOutput.AppendText("Eliminate Objects from Exclusion File:");
                            this.txtOutput.AppendText(Environment.NewLine);
                            fileWriter_err.WriteLine("======================================");
                            fileWriter_err.WriteLine("Eliminate Objects from Exclusion File:");
                            fileWriter_err.WriteLine("======================================");
                            if (File.Exists(Environment.CurrentDirectory+"\\"+this.textBox_ExclusionFile.Text))
                            {
                                //Lees alle objecten uit exclusion file in
                                string filepath = this.textBox_ExclusionFile.Text;
                                StreamReader file = new StreamReader(Environment.CurrentDirectory + "\\"+filepath);
                                string line;
                                ArrayList objs = new ArrayList();
                                while ((line = file.ReadLine()) != null)
                                {
                                    if (line.Substring(0,1) != "#")
                                    {
                                        objs.Add(line);
                                    }
                                }
                                for (int i = 0; i < fulloutputfile.Count; i++)
                                {
                                    string obj=fulloutputfile[i].ToString().Split(',')[0];
                                    if ((obj.Substring(0, 1) != "#") || (obj.Substring(0, 1) != "!"))
                                    {
                                        for (int j = 0; j < objs.Count; j++)
                                        {
                                            if (objs[j].ToString().Split(';')[0].Equals(obj))
                                            {
                                                this.txtOutput.AppendText(fulloutputfile[i].ToString());
                                                this.txtOutput.AppendText(Environment.NewLine);
                                                fileWriter_err.WriteLine(fulloutputfile[i]);
                                                string writetoexclusionfile_line = fulloutputfile[i].ToString();           //de lijn zonder % vanvoor
                                                fulloutputfile[i] = "%" + fulloutputfile[i];                    //Vlag voor objecten uit exclusion list: %
                                                if (this.checkBox_WriteToExclusionFiles.Checked)
                                                {
                                                    string exclusionfilename = objs[j].ToString().Split(';')[1];
                                                    if (exclusionfilename.Equals("NOFILE"))
                                                    {
                                                        this.txtOutput.AppendText("No File flag: not writing to any file.");
                                                        this.txtOutput.AppendText(Environment.NewLine);
                                                        fileWriter_err.WriteLine("No File flag: not writing to any file.");
                                                    }
                                                    if (File.Exists(exclusionfilename) && (exclusionfilename!="NOFILE"))
                                                    {
                                                        StreamReader fileReaderExclusionFile = new StreamReader(exclusionfilename);
                                                        Boolean doubleline=false;
                                                        while ((line = fileReaderExclusionFile.ReadLine()) != null)
                                                        {
                                                            if (line.Equals(writetoexclusionfile_line))
                                                            {
                                                                doubleline = true;
                                                                this.txtOutput.AppendText("ERROR: Line already exist in Exclusion File: " + exclusionfilename + "...Skipping...");
                                                                this.txtOutput.AppendText(Environment.NewLine);
                                                                fileWriter_err.WriteLine("ERROR: Line already exist in Exclusion File:  " + exclusionfilename+ "...Skipping...");
                                                            }
                                                        }
                                                        fileReaderExclusionFile.Dispose();
                                                        fileReaderExclusionFile.Close();
                                                        fileReaderExclusionFile = null;

                                                        if (doubleline == false)
                                                        {
                                                            StreamWriter fileWriterExclusionFile = new StreamWriter(exclusionfilename, true);
                                                            fileWriterExclusionFile.WriteLine(writetoexclusionfile_line);
                                                            fileWriterExclusionFile.Flush();
                                                            fileWriterExclusionFile.Close();
                                                            fileWriterExclusionFile = null;
                                                            this.txtOutput.AppendText("Written to file: " + exclusionfilename);
                                                            this.txtOutput.AppendText(Environment.NewLine);
                                                            fileWriter_err.WriteLine("Written to file: " + exclusionfilename);
                                                        }

                                                    }
                                                    else
                                                    {
                                                        this.txtOutput.AppendText("ERROR: File does not exist: " + exclusionfilename);
                                                        this.txtOutput.AppendText(Environment.NewLine);
                                                        fileWriter_err.WriteLine("ERROR: File does not exist: " + exclusionfilename);
                                                    }

                                                }
                                            }
                                        }
                                    }
                                }
                                file.Close();
                                file.Dispose();
                                file = null;
                            }
                            else
                            {
                                this.txtOutput.AppendText("Exclusion File does not exist.");
                                this.txtOutput.AppendText(Environment.NewLine);
                            }
                        }

                        if (this.checkBox_EliminateMagnitudeError.Checked)
                        {
                            this.txtOutput.AppendText("Eliminate Magnitude Error too High:");
                            this.txtOutput.AppendText(Environment.NewLine);

                            string str_maxmagerr = this.textBox_MagnitudeError.Text;
                            double maxmagerr = Double.Parse(str_maxmagerr);

                            ArrayList detectedobjs = new ArrayList();


                            for (int i = 0; i < fulloutputfile.Count; i++)
                            {
                                if (fulloutputfile[i].ToString().Substring(0, 1) != "#")
                                {
                                    string str_magerr = fulloutputfile[i].ToString().Split(',')[3];
                                    double magerr = Double.Parse(str_magerr);
                                    if (magerr > maxmagerr)
                                    {
                                        detectedobjs.Add(fulloutputfile[i]);
                                        fulloutputfile[i] = "§" + fulloutputfile[i];                            //Vlag voor te hoge magnitudes: §
                                    }
                                }
                            }

                            if (detectedobjs.Count > 0)
                            {
                                fileWriter_err.WriteLine("===================================");
                                fileWriter_err.WriteLine("Eliminate Magnitude Error too High:");
                                fileWriter_err.WriteLine("===================================");
                                foreach(string s in detectedobjs)
                                {
                                    fileWriter_err.WriteLine(s);
                                }
                            }

                        }

                        this.txtOutput.AppendText("Write away to end file...");
                        this.txtOutput.AppendText(Environment.NewLine);
                        foreach (string s in fulloutputfile)
                        {
                            this.txtOutput.AppendText(s);
                            this.txtOutput.AppendText(Environment.NewLine);
                            //schrijf alle lijnen weg in de array, behalve de gevlagde lijnen
                            if (!s.Substring(0, 1).Equals("!") && !s.Substring(0, 1).Equals("%") && !s.Substring(0, 1).Equals("§"))
                            {
                                fileWriter.WriteLine(s);
                            }
                        }


                        fileWriter.Flush();
                        fileWriter.Close();
                        this.txtOutput.AppendText("Generation DONE !\n");
                        fileWriter = null;
                        outputFile = null;
                        fileWriter_err.Flush();
                        fileWriter_err.Close();
                        fileWriter_err = null;



                    }
                    else
                    {
                        this.txtOutput.AppendText("No AAVSO report files found.  Exiting.\n");
                    }
                    files = null;

                }
                else
                {
                    this.txtOutput.AppendText("Supplied directory does not exist.  Exiting...\n");
                }

                DateTime endtime = DateTime.Now;
                double seconds = (endtime - starttime).TotalSeconds;
                this.label_Status.Text = "Done!  Time needed (seconds): " + Convert.ToInt32(seconds);
                this.label_Status.ForeColor = Color.Green;
                this.label_Status.Refresh();
            }
            catch (Exception ex)
            {
                this.txtOutput.AppendText("ERROR: \n"+ex.ToString());
                this.label_Status.Text = "ERROR!";
                this.label_Status.ForeColor = Color.Red;
                this.label_Status.Refresh();
            }

        }

        private void label1_MouseHover(object sender, EventArgs e)
        {
            this.toolTip1.SetToolTip(this.label1, "Filename of mass AAVSO report to generate.");
        }

        private void label2_MouseHover(object sender, EventArgs e)
        {
            this.toolTip1.SetToolTip(this.label2, "Directory of all the AAVSO reports that needs to be merged.");
        }

        private void label4_MouseHover(object sender, EventArgs e)
        {
            this.toolTip1.SetToolTip(this.label4, "Wildcard to detect the AAVSO report files.");
        }
    }
}
